﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  06/26/13    Created 
//
//---------------------------------------------------------------------------

namespace System.Drawing.Imaging
{
    using System.Runtime.InteropServices;

    /// <summary>
    /// Defines SGI constants and shared methods for the <see cref="SGIDecoder"/> and <see cref="SGIEncoder"/> classes.
    /// </summary>
    public abstract class SGICodec
    {
        /// <summary>
        /// SGI file type magic number.
        /// </summary>
        protected const short MAGIC_NUMBER = 474;

        /// <summary>
        /// Offsets the pointer position from an BGRA scheme used in bitmap to an RGBA scheme used in SGI.
        /// </summary>
        /// <param name="format">The format scheme used.</param>
        /// <param name="z">The bitmap channel to offset.</param>
        /// <param name="buffer_ps">A pointer to bitmap pixels.</param>
        /// <returns>A pointer to bitmap pixels offset to an SGI scheme.</returns>
        protected static unsafe byte* OffsetChannelPointer(SGIFormat format, int z, byte* buffer_ps)
        {
            switch (format)
            {
                case SGIFormat.RGBA:
                case SGIFormat.RGB:
                    //Bitmap pixels are stored BGRA while SGI images are stored RGBA
                    //so the buffer pointer must be offset RGB but not A:
                    if (z < 3)
                        return buffer_ps + 2 - z;
                    else
                        return buffer_ps + 3;

                case SGIFormat.BWA:
                    //Offset alpha channel:
                    if (z == 1)
                        return buffer_ps + 3;
                    break;
            }

            return buffer_ps;
        }
    }
}
